from datetime import datetime
from enum import IntEnum
from typing import Literal, Optional, Union

from pydantic import BaseModel, Extra, Field


class Mode(IntEnum):
    unknown = 0
    no_fix = 1
    two_d_fix = 2
    three_d_fix = 3


class WatchConfig(BaseModel):
    class_: Literal["WATCH"] = Field("WATCH", alias="class")
    enable: bool = True
    json_: bool = Field(True, alias="json")
    split24: bool = False
    raw: int = 0

    class Config:
        extra = Extra.allow


class Version(BaseModel):
    class_: Literal["VERSION"] = Field(alias="class")
    release: str
    rev: str
    proto_major: int
    proto_minor: int

    @property
    def proto(self) -> tuple[int, int]:
        return self.proto_major, self.proto_minor


class Device(BaseModel):
    class_: Literal["DEVICE"] = Field(alias="class")
    path: str
    driver: str
    subtype: str
    activated: datetime
    flags: int
    native: int
    bps: int
    parity: str
    stopbits: int
    cycle: float
    mincycle: float


class Devices(BaseModel):
    class_: Literal["DEVICES"] = Field(alias="class")
    devices: list[Device]


class TPV(BaseModel):
    class_: Literal["TPV"] = Field(alias="class")
    device: str
    mode: Mode
    time: datetime
    ept: float
    lat: float
    lon: float
    altHAE: float
    altMSL: float
    alt: float
    epx: float
    epy: float
    epv: float
    track: Optional[float]
    magtrack: Optional[float]
    magvar: float
    speed: float
    climb: float
    eps: float
    epc: float
    geoidSep: float
    eph: float
    sep: float


class PRN(BaseModel):
    PRN: int
    el: float
    az: float
    ss: float
    used: bool
    gnssid: int
    svid: int


class Sky(BaseModel):
    class_: Literal["SKY"] = Field(alias="class")
    device: str
    xdop: float
    ydop: float
    vdop: float
    tdop: float
    hdop: float
    gdop: float
    nSat: int
    uSat: int
    satellites: list[PRN]


class Poll(BaseModel):
    class_: Literal["POLL"] = Field(alias="class")
    time: datetime
    active: int
    tpv: list[TPV]
    sky: list[Sky]


class Response(BaseModel):
    __root__: Union[Poll, Sky, TPV, Devices, Version, WatchConfig] = Field(discriminator="class_")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['asyncio_gpsd_client']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.2,<2.0.0']

setup_kwargs = {
    'name': 'asyncio-gpsd-client',
    'version': '0.2.0',
    'description': 'asyncio compatible gpsd client',
    'long_description': '# Asycio Gpsd Client\n\n# Install\n\n```shell\npip install asyncio-gpsd-client\n```\n\n# Usage\n\n```python\nimport asyncio\n\nfrom asyncio_gpsd_client import GpsdClient\n\nHOST = "127.0.0.1"\nPORT = 2947\n\n\nasync def main():\n    async with GpsdClient(HOST, PORT) as client:\n        print(await client.poll())  # Get gpsd POLL response\n        while True:\n            print(await client.get_result())  # Get gpsd TPV responses\n\n\nasyncio.run(main())\n```\n',
    'author': 'Zachary Juang',
    'author_email': 'zachary822@me.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import asyncio

from asyncio_gpsd_client.exceptions import GpsdClientError
from asyncio_gpsd_client.schemas import Devices, Response, Version, WatchConfig

POLL = "?POLL;\r\n"
WATCH = "?WATCH={}\r\n"


class GpsdClient:
    reader: asyncio.StreamReader
    writer: asyncio.StreamWriter
    version: Version
    devices: Devices
    watch: WatchConfig

    def __init__(self, host: str = "127.0.0.1", port: int = 2947, watch_config: WatchConfig = WatchConfig()):
        self.host = host
        self.port = port

        self.watch_config = watch_config

    def __getattr__(self, item):
        if item in ("reader", "writer"):
            raise GpsdClientError("Initiate connection with client.connect()")
        raise AttributeError(item)

    async def connect(self):
        self.reader, self.writer = await asyncio.open_connection(self.host, self.port)

        self.writer.write(WATCH.format(self.watch_config.json(by_alias=True)).encode())
        await self.writer.drain()

        self.version = await self.get_result()
        self.devices = await self.get_result()
        self.watch = await self.get_result()

    async def close(self):
        self.writer.close()
        await self.writer.wait_closed()

    async def get_result(self):
        return Response.parse_raw(await self.reader.readline()).__root__

    async def poll(self):
        self.writer.write(POLL.encode())
        await self.writer.drain()
        return await self.get_result()

    async def __aenter__(self):
        await self.connect()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.close()

    def __aiter__(self):
        return self

    async def __anext__(self):
        result = await self.get_result()
        if result.class_ == "TPV":
            return result
        if result.class_ == "SKY":
            self.sky = result
        return await anext(self)
